@@ -28,12 +28,12 @@
    not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "addrxlat-priv.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 
-#include "addrxlat-priv.h"
-
 /** Maximum length of the static error message. */
 #define ERRBUF	64
 
@@ -56,24 +56,19 @@ init_cache(struct read_cache *cache)
 
 /**  Clean up the read cache.
  * @param cache  Read cache.
- * @param cb     Callback definitions.
  *
- * Release all cached pages using @c put_page function from the
- * provided callback definition.
+ * Release all cached pages.
  */
 static void
-cleanup_cache(struct read_cache *cache, const addrxlat_cb_t *cb)
+cleanup_cache(struct read_cache *cache)
 {
 	struct read_cache_slot *slot;
 
-	if (!cb->put_page)
-		return;
-
 	slot = &cache->slot[0];
 	do {
 		addrxlat_buffer_t *buf = &slot->buffer;
 		if (buf->size)
-			cb->put_page(cb->data, buf);
+			buf->put_page(buf);
 	} while (++slot < &cache->slot[READ_CACHE_SLOTS]);
 }
 
@@ -101,13 +96,21 @@ touch_cache_slot(struct read_cache *cache, struct read_cache_slot *slot)
 	cache->mru = slot;
 }
 
+/** Default put-page callback.
+ * @param buf  Read buffer metadata (unused).
+ */
+static void
+def_put_page_cb(const addrxlat_buffer_t *buf)
+{
+}
+
 /** Get a cache slot for a given address.
  * @param      ctx   Address translation context.
  * @param      addr  Desired address.
  * @param[out] pbuf  Buffer (updated on success).
  * @returns          Error status.
  */
-static addrxlat_status
+addrxlat_status
 get_cache_buf(addrxlat_ctx_t *ctx, const addrxlat_fulladdr_t *addr,
 	      addrxlat_buffer_t **pbuf)
 {
@@ -127,18 +130,24 @@ get_cache_buf(addrxlat_ctx_t *ctx, const addrxlat_fulladdr_t *addr,
 	slot = ctx->cache.mru->prev;
 
 	/* Free up the slot if necessary */
-	if (slot->buffer.size && ctx->cb.put_page)
-		ctx->cb.put_page(ctx->cb.data, &slot->buffer);
+	if (slot->buffer.size)
+		slot->buffer.put_page(&slot->buffer);
 
 	/* Get the new page */
 	slot->buffer.addr = *addr;
-	status = ctx->cb.get_page(ctx->cb.data, &slot->buffer);
+	slot->buffer.ptr = NULL;
+	slot->buffer.put_page = def_put_page_cb;
+	status = ctx->cb->get_page(ctx->cb, &slot->buffer);
 	if (status != ADDRXLAT_OK) {
 		slot->buffer.size = 0;
 		return status;
 	}
 
  out:
+	if (!slot->buffer.ptr)
+		return set_error(ctx, ADDRXLAT_ERR_NODATA,
+				 "Infinite read recursion");
+
 	*pbuf = &slot->buffer;
 	touch_cache_slot(&ctx->cache, slot);
 	return ADDRXLAT_OK;
@@ -185,12 +194,121 @@ bury_cache_buffer(struct read_cache *cache, const addrxlat_fulladdr_t *addr)
 	} while (++slot < &cache->slot[READ_CACHE_SLOTS]);
 }
 
+/** Missing callback handler.
+ * @param cb    Default callback definition.
+ * @param name  Name of the callback.
+ * @returns     Error status.
+ */
+static addrxlat_status
+no_callback(const addrxlat_cb_t *cb, const char *name)
+{
+	addrxlat_ctx_t *ctx = cb->priv;
+	return set_error(ctx, ADDRXLAT_ERR_NODATA,
+			 "No %s callback", name);
+}
+
+/** Default get-page callback.
+ * @param cb    This callback definition.
+ * @param buf   Read buffer metadata (unused).
+ * @returns     Error status.
+ */
+static addrxlat_status
+def_get_page_cb(const addrxlat_cb_t *cb, addrxlat_buffer_t *buf)
+{
+	return no_callback(cb, "get_page");
+}
+
+/** Default read capabilities callback.
+ * @param cb    This callback definition.
+ * @returns     Read capabilities.
+ */
+static unsigned long
+def_read_caps_cb(const addrxlat_cb_t *cb)
+{
+	return 0;
+}
+
+/** Default register value callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Register name.
+ * @param val[out]  Register value.
+ * @returns         Error status.
+ */
+static addrxlat_status
+def_reg_value_cb(const addrxlat_cb_t *cb, const char *name,
+		 addrxlat_addr_t *val)
+{
+	return no_callback(cb, "reg_value");
+}
+
+/** Default symbol value callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Symbol name.
+ * @param val[out]  Symbol value (e.g. address of a variable).
+ * @returns         Error status.
+ */
+static addrxlat_status
+def_sym_value_cb(const addrxlat_cb_t *cb, const char *name,
+		 addrxlat_addr_t *val)
+{
+	return no_callback(cb, "sym_value");
+}
+
+/** Default symbol size callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Name of a symbol or data type.
+ * @param val[out]  Symbol size, @c sizeof(name).
+ * @returns         Error status.
+ */
+static addrxlat_status
+def_sym_sizeof_cb(const addrxlat_cb_t *cb, const char *name,
+		  addrxlat_addr_t *val)
+{
+	return no_callback(cb, "sym_sizeof");
+}
+
+/** Default element offset callback.
+ * @param cb        This callback definition.
+ * @param obj[in]   Name of the container object (e.g. of a @c struct).
+ * @param elem[in]  Name of the element (e.g. a structure field).
+ * @param val[out]  Offset of @p elem within @p obj, @c offsetof(obj, elem).
+ * @returns         Error status.
+ */
+static addrxlat_status
+def_sym_offsetof_cb(const addrxlat_cb_t *cb, const char *obj,
+		    const char *elem, addrxlat_addr_t *val)
+{
+	return no_callback(cb, "sym_offsetof");
+}
+
+/** Default number value callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Name of the numeric parameter.
+ * @param val[out]  Parameter value.
+ * @returns         Error status.
+ */
+static addrxlat_status
+def_num_value_cb(const addrxlat_cb_t *cb, const char *name,
+		 addrxlat_addr_t *val)
+{
+	return no_callback(cb, "sym_offsetof");
+}
+
 addrxlat_ctx_t *
 addrxlat_ctx_new(void)
 {
 	addrxlat_ctx_t *ctx = calloc(1, sizeof(addrxlat_ctx_t) + ERRBUF);
 	if (ctx) {
 		ctx->refcnt = 1;
+		ctx->cb = &ctx->def_cb;
+		ctx->def_cb.priv = ctx;
+		ctx->def_cb.get_page = def_get_page_cb;
+		ctx->def_cb.read_caps = def_read_caps_cb;
+		ctx->def_cb.reg_value = def_reg_value_cb;
+		ctx->def_cb.sym_value = def_sym_value_cb;
+		ctx->def_cb.sym_sizeof = def_sym_sizeof_cb;
+		ctx->def_cb.sym_offsetof = def_sym_offsetof_cb;
+		ctx->def_cb.num_value = def_num_value_cb;
 		init_cache(&ctx->cache);
 		err_init(&ctx->err, ERRBUF);
 	}
@@ -208,7 +326,13 @@ addrxlat_ctx_decref(addrxlat_ctx_t *ctx)
 {
 	unsigned long refcnt = --ctx->refcnt;
 	if (!refcnt) {
-		cleanup_cache(&ctx->cache, &ctx->cb);
+		cleanup_cache(&ctx->cache);
+		addrxlat_cb_t *p = (addrxlat_cb_t *)ctx->cb;
+		while (p != &ctx->def_cb) {
+			const addrxlat_cb_t *next = p->next;
+			free(p);
+			p = (addrxlat_cb_t *)next;
+		}
 		err_cleanup(&ctx->err);
 		free(ctx);
 	}
@@ -232,34 +356,143 @@ addrxlat_ctx_get_errmsg(addrxlat_ctx_t *ctx)
 	return &ctx->err;
 }
 
-void
-addrxlat_ctx_set_cb(addrxlat_ctx_t *ctx, const addrxlat_cb_t *cb)
+/** Call the next get-page callback.
+ * @param cb    This callback definition.
+ * @param buf   Read buffer metadata.
+ * @returns     Error status.
+ */
+static addrxlat_status
+next_get_page_cb(const addrxlat_cb_t *cb, addrxlat_buffer_t *buf)
 {
-	addrxlat_cb_hook_fn *hook = ctx->cb.cb_hook;
-	void *data = ctx->cb.data;
-	ctx->cb = ctx->orig_cb = *cb;
-	if (hook)
-		hook(data, &ctx->cb);
+	return cb->next->get_page(cb->next, buf);
+}
+
+/** Call the next read capabilities callback.
+ * @param cb    This callback definition.
+ * @returns     Read capabilities.
+ */
+static unsigned long
+next_read_caps_cb(const addrxlat_cb_t *cb)
+{
+	return cb->next->read_caps(cb->next);
+}
+
+/** Call the next register value callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Register name.
+ * @param val[out]  Register value.
+ * @returns         Error status.
+ */
+static addrxlat_status
+next_reg_value_cb(const addrxlat_cb_t *cb, const char *name,
+		  addrxlat_addr_t *val)
+{
+	return cb->next->reg_value(cb, name, val);
+}
+
+/** Call the next symbol value callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Symbol name.
+ * @param val[out]  Symbol value (e.g. address of a variable).
+ * @returns         Error status.
+ */
+static addrxlat_status
+next_sym_value_cb(const addrxlat_cb_t *cb, const char *name,
+		  addrxlat_addr_t *val)
+{
+	return cb->next->sym_value(cb, name, val);
+}
+
+/** Call the next symbol size callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Name of a symbol or data type.
+ * @param val[out]  Symbol size, @c sizeof(name).
+ * @returns         Error status.
+ */
+static addrxlat_status
+next_sym_sizeof_cb(const addrxlat_cb_t *cb, const char *name,
+		   addrxlat_addr_t *val)
+{
+	return cb->next->sym_sizeof(cb, name, val);
+}
+
+/** Call the next element offset callback.
+ * @param cb        This callback definition.
+ * @param obj[in]   Name of the container object (e.g. of a @c struct).
+ * @param elem[in]  Name of the element (e.g. a structure field).
+ * @param val[out]  Offset of @p elem within @p obj, @c offsetof(obj, elem).
+ * @returns         Error status.
+ */
+static addrxlat_status
+next_sym_offsetof_cb(const addrxlat_cb_t *cb, const char *obj,
+		     const char *elem, addrxlat_addr_t *val)
+{
+	return cb->next->sym_offsetof(cb, obj, elem, val);
+}
+
+/** Call the next number value callback.
+ * @param cb        This callback definition.
+ * @param name[in]  Name of the numeric parameter.
+ * @param val[out]  Parameter value.
+ * @returns         Error status.
+ */
+static addrxlat_status
+next_num_value_cb(const addrxlat_cb_t *cb, const char *name,
+		  addrxlat_addr_t *val)
+{
+	return cb->next->num_value(cb, name, val);
+}
+
+addrxlat_cb_t *
+addrxlat_ctx_add_cb(addrxlat_ctx_t *ctx)
+{
+	addrxlat_cb_t *cb;
+
+	cb = malloc(sizeof *cb);
+	if (!cb)
+		return cb;
+
+	cb->next = ctx->cb;
+	cb->priv = NULL;
+	cb->get_page = next_get_page_cb;
+	cb->read_caps = next_read_caps_cb;
+	cb->reg_value = next_reg_value_cb;
+	cb->sym_value = next_sym_value_cb;
+	cb->sym_sizeof = next_sym_sizeof_cb;
+	cb->sym_offsetof = next_sym_offsetof_cb;
+	cb->num_value = next_num_value_cb;
+
+	ctx->cb = cb;
+
+	return cb;
+}
+
+void
+addrxlat_ctx_del_cb(addrxlat_ctx_t *ctx, addrxlat_cb_t *cb)
+{
+	const addrxlat_cb_t **pprev = &ctx->cb;
+	const addrxlat_cb_t *p = ctx->cb;
+
+	while (p && p != cb) {
+		pprev = (const addrxlat_cb_t **) &p->next;
+		p = p->next;
+	}
+	if (p) {
+		*pprev = cb->next;
+		free(cb);
+	}
 }
 
 const addrxlat_cb_t *
 addrxlat_ctx_get_cb(const addrxlat_ctx_t *ctx)
 {
-	return &ctx->orig_cb;
+	return ctx->cb;
 }
 
-addrxlat_cb_t *
-addrxlat_ctx_get_ecb(addrxlat_ctx_t *ctx)
-{
-	return &ctx->cb;
-}
+DEFINE_ALIAS(addrspace_name);
 
-/** Get the (string) name of an address space.
- * @param as  Address space.
- * @returns   The (human-readable) name of the address space.
- */
-static const char *
-addrspace_name(addrxlat_addrspace_t as)
+const char *
+addrxlat_addrspace_name(addrxlat_addrspace_t as)
 {
 	switch (as) {
 	case ADDRXLAT_KPHYSADDR:	return "KPHYSADDR";
@@ -270,9 +503,27 @@ addrspace_name(addrxlat_addrspace_t as)
 	}
 }
 
-/** Common format string for missing read callback. */
-static const char read_nocb_fmt[] =
-	"No read callback for %s";
+/** Verify that an address can be read without translation.
+ * @param      ctx  Address translation context.
+ * @param[in] addr  Full address of the data.
+ * @returns         @c true if readable.
+ */
+bool
+direct_read_ok(addrxlat_ctx_t *ctx, const addrxlat_fulladdr_t *addr)
+{
+	unsigned long read_caps = ctx->cb->read_caps(ctx->cb);
+	addrxlat_buffer_t *buffer;
+
+	if (! (read_caps & ADDRXLAT_CAPS(addr->as)))
+		return false;
+
+	if (get_cache_buf(ctx, addr, &buffer) != ADDRXLAT_OK) {
+		clear_error(ctx);
+		return false;
+	}
+
+	return true;
+}
 
 /** Common format string for read callback failures. */
 static const char read_err_fmt[] =
@@ -328,13 +579,11 @@ read32(addrxlat_step_t *step, const addrxlat_fulladdr_t *addr, uint32_t *val,
        const char *what)
 {
 	addrxlat_ctx_t *ctx = step->ctx;
+	unsigned long read_caps;
 	addrxlat_status status;
 
-	if (!ctx->cb.get_page)
-		return set_error(ctx, ADDRXLAT_ERR_NODATA, read_nocb_fmt,
-				 addrspace_name(addr->as));
-
-	if (ctx->cb.read_caps & ADDRXLAT_CAPS(addr->as)) {
+	read_caps = ctx->cb->read_caps(ctx->cb);
+	if (read_caps & ADDRXLAT_CAPS(addr->as)) {
 		status = do_read32(ctx, addr, val);
 	} else {
 		addrxlat_op_ctl_t ctl;
@@ -344,13 +593,13 @@ read32(addrxlat_step_t *step, const addrxlat_fulladdr_t *addr, uint32_t *val,
 		ctl.sys = step->sys;
 		ctl.op = read32_op;
 		ctl.data = &param;
-		ctl.caps = ctx->cb.read_caps;
+		ctl.caps = read_caps;
 		status = internal_op(&ctl, addr);
 	}
 
 	if (status != ADDRXLAT_OK)
 		return set_error(ctx, status, read_err_fmt, 32, what,
-				 addrspace_name(addr->as), addr->addr);
+				 internal_addrspace_name(addr->as), addr->addr);
 
 	return ADDRXLAT_OK;
 }
@@ -400,13 +649,11 @@ read64(addrxlat_step_t *step, const addrxlat_fulladdr_t *addr, uint64_t *val,
        const char *what)
 {
 	addrxlat_ctx_t *ctx = step->ctx;
+	unsigned long read_caps;
 	addrxlat_status status;
 
-	if (!ctx->cb.get_page)
-		return set_error(ctx, ADDRXLAT_ERR_NODATA, read_nocb_fmt,
-				 addrspace_name(addr->as));
-
-	if (ctx->cb.read_caps & ADDRXLAT_CAPS(addr->as)) {
+	read_caps = ctx->cb->read_caps(ctx->cb);
+	if (read_caps & ADDRXLAT_CAPS(addr->as)) {
 		status = do_read64(ctx, addr, val);
 	} else {
 		addrxlat_op_ctl_t ctl;
@@ -416,13 +663,13 @@ read64(addrxlat_step_t *step, const addrxlat_fulladdr_t *addr, uint64_t *val,
 		ctl.sys = step->sys;
 		ctl.op = read64_op;
 		ctl.data = &param;
-		ctl.caps = ctx->cb.read_caps;
+		ctl.caps = read_caps;
 		status = internal_op(&ctl, addr);
 	}
 
 	if (status != ADDRXLAT_OK)
 		return set_error(ctx, status, read_err_fmt, 64, what,
-				 addrspace_name(addr->as), addr->addr);
+				 internal_addrspace_name(addr->as), addr->addr);
 
 	return ADDRXLAT_OK;
 }
@@ -438,22 +685,12 @@ read64(addrxlat_step_t *step, const addrxlat_fulladdr_t *addr, uint64_t *val,
 addrxlat_status
 get_reg(addrxlat_ctx_t *ctx, const char *name, addrxlat_addr_t *val)
 {
-	addrxlat_sym_t sym;
-	addrxlat_status status;
-
-	if (!ctx->cb.sym)
-		return set_error(ctx, ADDRXLAT_ERR_NODATA,
-				 "No symbolic information callback");
-
-	sym.type = ADDRXLAT_SYM_REG;
-	sym.args[0] = name;
-	status = ctx->cb.sym(ctx->cb.data, &sym);
-	if (status != ADDRXLAT_OK)
-		return set_error(ctx, status,
-				 "Cannot read register \"%s\"", sym.args[0]);
-
-	*val = sym.val;
-	return status;
+	addrxlat_status status =
+		ctx->cb->reg_value(ctx->cb, name, val);
+	return status != ADDRXLAT_OK
+		? set_error(ctx, status,
+			    "Cannot read register \"%s\"", name)
+		: status;
 }
 
 /** Resolve a symbol value.
@@ -467,22 +704,12 @@ get_reg(addrxlat_ctx_t *ctx, const char *name, addrxlat_addr_t *val)
 addrxlat_status
 get_symval(addrxlat_ctx_t *ctx, const char *name, addrxlat_addr_t *val)
 {
-	addrxlat_sym_t sym;
-	addrxlat_status status;
-
-	if (!ctx->cb.sym)
-		return set_error(ctx, ADDRXLAT_ERR_NODATA,
-				 "No symbolic information callback");
-
-	sym.type = ADDRXLAT_SYM_VALUE;
-	sym.args[0] = name;
-	status = ctx->cb.sym(ctx->cb.data, &sym);
-	if (status != ADDRXLAT_OK)
-		return set_error(ctx, status,
-				 "Cannot resolve \"%s\"", sym.args[0]);
-
-	*val = sym.val;
-	return status;
+	addrxlat_status status =
+		ctx->cb->sym_value(ctx->cb, name, val);
+	return status != ADDRXLAT_OK
+		? set_error(ctx, status,
+			    "Cannot resolve \"%s\"", name)
+		: status;
 }
 
 /** Get the size of a symbol or type.
@@ -496,22 +723,11 @@ get_symval(addrxlat_ctx_t *ctx, const char *name, addrxlat_addr_t *val)
 addrxlat_status
 get_sizeof(addrxlat_ctx_t *ctx, const char *name, addrxlat_addr_t *sz)
 {
-	addrxlat_sym_t sym;
-	addrxlat_status status;
-
-	if (!ctx->cb.sym)
-		return set_error(ctx, ADDRXLAT_ERR_NODATA,
-				 "No symbolic information callback");
-
-	sym.type = ADDRXLAT_SYM_SIZEOF;
-	sym.args[0] = name;
-	status = ctx->cb.sym(ctx->cb.data, &sym);
-	if (status != ADDRXLAT_OK)
-		return set_error(ctx, status, "Cannot get sizeof(%s)",
-				 sym.args[0]);
-
-	*sz = sym.val;
-	return status;
+	addrxlat_status status =
+		ctx->cb->sym_sizeof(ctx->cb, name, sz);
+	return status != ADDRXLAT_OK
+		? set_error(ctx, status, "Cannot get sizeof(%s)", name)
+		: status;
 }
 
 /** Get the relative offset of a member inside a type.
@@ -527,23 +743,12 @@ addrxlat_status
 get_offsetof(addrxlat_ctx_t *ctx, const char *type, const char *memb,
 	     addrxlat_addr_t *off)
 {
-	addrxlat_sym_t sym;
-	addrxlat_status status;
-
-	if (!ctx->cb.sym)
-		return set_error(ctx, ADDRXLAT_ERR_NODATA,
-				 "No symbolic information callback");
-
-	sym.type = ADDRXLAT_SYM_OFFSETOF;
-	sym.args[0] = type;
-	sym.args[1] = memb;
-	status = ctx->cb.sym(ctx->cb.data, &sym);
-	if (status != ADDRXLAT_OK)
-		return set_error(ctx, status, "Cannot get offsetof(%s, %s)",
-				 sym.args[0], sym.args[1]);
-
-	*off = sym.val;
-	return status;
+	addrxlat_status status =
+		ctx->cb->sym_offsetof(ctx->cb, type, memb, off);
+	return status != ADDRXLAT_OK
+		? set_error(ctx, status, "Cannot get offsetof(%s, %s)",
+			    type, memb)
+		: status;
 }
 
 /** Resolve a number value.
@@ -557,61 +762,11 @@ get_offsetof(addrxlat_ctx_t *ctx, const char *type, const char *memb,
 addrxlat_status
 get_number(addrxlat_ctx_t *ctx, const char *name, addrxlat_addr_t *num)
 {
-	addrxlat_sym_t sym;
-	addrxlat_status status;
-
-	if (!ctx->cb.sym)
-		return set_error(ctx, ADDRXLAT_ERR_NODATA,
-				 "No symbolic information callback");
-
-	sym.type = ADDRXLAT_SYM_NUMBER;
-	sym.args[0] = name;
-	status = ctx->cb.sym(ctx->cb.data, &sym);
-	if (status != ADDRXLAT_OK)
-		return set_error(ctx, status, "Cannot get number(%s)",
-				 sym.args[0]);
-
-	*num = sym.val;
-	return status;
-}
-
-
-/** Get the first successfuly resolved value from a specifier list.
- * @param      ctx   Address translation context.
- * @param      spec  Vector of specifiers.
- * @param[out] addr  Symbol full address, returned on sucess.
- * @returns          Error status.
- *
- * The symbol is resolved using a user-supplied callback.
- */
-addrxlat_status
-get_first_sym(addrxlat_ctx_t *ctx, const struct sym_spec *spec,
-	      addrxlat_fulladdr_t *addr)
-{
-	addrxlat_status status = ADDRXLAT_ERR_NODATA;
-
-	if (!ctx->cb.sym)
-		return set_error(ctx, status,
-				 "No symbolic information callback");
-
-	while (spec->type != ADDRXLAT_SYM_NONE) {
-		addrxlat_sym_t sym;
-		sym.type = spec->type;
-		sym.args[0] = spec->name;
-		status = ctx->cb.sym(ctx->cb.data, &sym);
-		if (status == ADDRXLAT_OK) {
-			addr->addr = sym.val;
-			addr->as = spec->as;
-			return status;
-		} else if (status != ADDRXLAT_ERR_NODATA)
-			break;
-
-		clear_error(ctx);
-		++spec;
-	}
-
-	return set_error(ctx, status,
-			 "Cannot resolve \"%s\"", spec->name);
+	addrxlat_status status =
+		ctx->cb->num_value(ctx->cb, name, num);
+	return status != ADDRXLAT_OK
+		? set_error(ctx, status, "Cannot get number(%s)", name)
+		: status;
 }
 
 DEFINE_ALIAS(ctx_err);
