@@ -26,14 +26,15 @@
    not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "testutil.h"
+
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <getopt.h>
 #include <libkdumpfile/addrxlat.h>
 
-#include "testutil.h"
-
 #define ALLOC_INC 32
 
 static size_t nentries;
@@ -49,10 +50,17 @@ find_entry(addrxlat_addr_t addr)
 	return NULL;
 }
 
-static addrxlat_status
-get_page(void *data, addrxlat_buffer_t *buf)
+static unsigned long
+read_caps(const addrxlat_cb_t *cb)
 {
-	addrxlat_ctx_t *ctx = data;
+	return (ADDRXLAT_CAPS(ADDRXLAT_MACHPHYSADDR) |
+		ADDRXLAT_CAPS(ADDRXLAT_KVADDR));
+}
+
+static addrxlat_status
+get_page(const addrxlat_cb_t *cb, addrxlat_buffer_t *buf)
+{
+	addrxlat_ctx_t *ctx = cb->priv;
 	addrxlat_lookup_elem_t *ent = find_entry(buf->addr.addr);
 	if (!ent)
 		return addrxlat_ctx_err(ctx, ADDRXLAT_ERR_NODATA, "No data");
@@ -92,15 +100,25 @@ add_entry(const char *spec)
 	}
 
 	entries[nentries].orig = addr;
-	entries[nentries].dest = htole64(val);
+	entries[nentries].dest = val;
 	++nentries;
 
 	return TEST_OK;
 }
 
+static void
+entries_htole64(void)
+{
+	size_t i;
+	for (i = 0; i < nentries; ++i)
+		entries[i].dest = htole64(entries[i].dest);
+}
+
 static int
 set_paging_form(addrxlat_paging_form_t *pf, const char *spec)
 {
+	addrxlat_pte_format_t fmt;
+	char *fmt_spec;
 	char *endp;
 
 	endp = strchr(spec, ':');
@@ -109,38 +127,28 @@ set_paging_form(addrxlat_paging_form_t *pf, const char *spec)
 		return TEST_ERR;
 	}
 
-	if (!strncasecmp(spec, "none:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_NONE;
-	else if (!strncasecmp(spec, "pfn32:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_PFN32;
-	else if (!strncasecmp(spec, "pfn64:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_PFN64;
-	else if (!strncasecmp(spec, "aarch64:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_AARCH64;
-	else if (!strncasecmp(spec, "ia32:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_IA32;
-	else if (!strncasecmp(spec, "ia32_pae:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_IA32_PAE;
-	else if (!strncasecmp(spec, "x86_64:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_X86_64;
-	else if (!strncasecmp(spec, "s390x:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_S390X;
-	else if (!strncasecmp(spec, "ppc64_linux_rpn30:", endp - spec + 1))
-		pf->pte_format = ADDRXLAT_PTE_PPC64_LINUX_RPN30;
-	else {
+	fmt_spec = alloca(endp - spec + 1);
+	memcpy(fmt_spec, spec, endp - spec);
+	fmt_spec[endp - spec] = '\0';
+
+	fmt = addrxlat_pte_format(fmt_spec);
+	if (fmt == ADDRXLAT_PTE_INVALID) {
 		fprintf(stderr, "Unknown PTE format: %s\n", spec);
 		return TEST_ERR;
 	}
+	pf->pte_format = fmt;
 
 	pf->nfields = 0;
-	do {
+	while (isdigit(*++endp)) {
 		if (pf->nfields >= ADDRXLAT_FIELDS_MAX) {
 			fprintf(stderr, "Too many paging levels!\n");
 			return TEST_ERR;
 		}
 		pf->fieldsz[pf->nfields++] =
-			strtoul(endp + 1, &endp, 0);
-	} while (*endp == ',');
+			strtoul(endp, &endp, 0);
+		if (*endp != ',')
+			break;
+	}
 
 	if (*endp) {
 		fprintf(stderr, "Invalid paging form: %s\n", spec);
@@ -335,11 +343,7 @@ main(int argc, char **argv)
 	unsigned long long vaddr;
 	char *endp;
 	addrxlat_ctx_t *ctx;
-	addrxlat_cb_t cb = {
-		.get_page = get_page,
-		.read_caps = (ADDRXLAT_CAPS(ADDRXLAT_MACHPHYSADDR) |
-			      ADDRXLAT_CAPS(ADDRXLAT_KVADDR))
-	};
+	addrxlat_cb_t *cb;
 	addrxlat_meth_t pgt, linear, lookup, memarr, *meth;
 	int opt;
 	unsigned long refcnt;
@@ -444,8 +448,11 @@ main(int argc, char **argv)
 		return TEST_ERR;
 	}
 
-	lookup.param.lookup.nelem = nentries;
-	lookup.param.lookup.tbl = entries;
+	if (meth == &lookup) {
+		lookup.param.lookup.nelem = nentries;
+		lookup.param.lookup.tbl = entries;
+	} else
+		entries_htole64();
 
 	ctx = addrxlat_ctx_new();
 	if (!ctx) {
@@ -453,9 +460,15 @@ main(int argc, char **argv)
 		rc = TEST_ERR;
 		goto out;
 	}
-	cb.data = ctx;
-	addrxlat_ctx_set_cb(ctx, &cb);
-
+	cb = addrxlat_ctx_add_cb(ctx);
+	if (!cb) {
+		perror("Cannot initialize address translation callbacks");
+		rc = TEST_ERR;
+		goto out;
+	}
+	cb->priv = ctx;
+	cb->get_page = get_page;
+	cb->read_caps = read_caps;
 	rc = do_xlat(ctx, meth, vaddr);
 
  out:
