@@ -26,15 +26,17 @@
    not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
+#include "testutil.h"
+
 #include <string.h>
 #include <endian.h>
 #include <sys/time.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <errno.h>
 
-#include "config.h"
-#include "testutil.h"
 #include "diskdump.h"
 
 #if USE_ZLIB
@@ -46,6 +48,9 @@
 #if USE_SNAPPY
 # include <snappy-c.h>
 #endif
+#if USE_ZSTD
+# include <zstd.h>
+#endif
 typedef int write_fn(FILE *);
 
 struct page_data_kdump {
@@ -65,8 +70,6 @@ struct page_data_kdump {
 #if USE_LZO
 	lzo_bytep lzo_wrkmem;
 #endif
-
-	unsigned long skip;
 };
 
 static endian_t be;
@@ -80,13 +83,12 @@ enum compress_method {
 	COMPRESS_ZLIB,
 	COMPRESS_LZO,
 	COMPRESS_SNAPPY,
+	COMPRESS_ZSTD,
 };
 
-struct data_block {
-	off_t filepos;
-	struct blob *blob;
-};
-
+static bool flattened;
+static unsigned long long flattened_type = MDF_TYPE_FLAT_HEADER;
+static unsigned long long flattened_version = MDF_VERSION_FLAT_HEADER;
 static char *arch_name;
 static unsigned long long compression;
 static char *signature;
@@ -117,17 +119,22 @@ static unsigned long long split;
 static unsigned long long start_pfn;
 static unsigned long long end_pfn;
 
-static struct data_block vmcoreinfo;
-static struct data_block notes;
-static struct data_block eraseinfo;
+static struct blob *vmcoreinfo;
+static struct blob *notes;
+static struct blob *eraseinfo;
 
 static char *vmcoreinfo_file;
 static char *note_file;
 static char *eraseinfo_file;
 static char *data_file;
 
+static unsigned long start_pdidx;
+
 static const struct param param_array[] = {
 	/* meta-data */
+	PARAM_YESNO("flattened", flattened),
+	PARAM_NUMBER("flattened.type", flattened_type),
+	PARAM_NUMBER("flattened.version", flattened_version),
 	PARAM_STRING("arch_name", arch_name),
 	PARAM_NUMBER("compression", compression),
 
@@ -211,16 +218,40 @@ set_default_params(void)
 	return TEST_OK;
 }
 
+static int
+write_chunk(FILE *f, off_t off, const void *ptr, size_t sz, const char *what)
+{
+	if (flattened) {
+		struct makedumpfile_data_header hdr = {
+			.offset = htobe64(off),
+			.buf_size = htobe64(sz),
+		};
+		if (fwrite(&hdr, sizeof hdr, 1, f) != 1) {
+			perror("flattened segment header");
+			return -1;
+		}
+	} else if (fseek(f, off, SEEK_SET) != 0) {
+		fprintf(stderr, "seek %s: %s\n", what, strerror(errno));
+		return -1;
+	}
+	if (fwrite(ptr, sz, 1, f) != 1) {
+		fprintf(stderr, "write %s: %s\n", what, strerror(errno));
+		return -1;
+	}
+	return 0;
+}
+
 static int
 writeheader_32(FILE *f)
 {
 	struct timeval tv;
 	struct disk_dump_header_32 hdr;
 	struct kdump_sub_header_32 subhdr;
+	off_t pos;
 
 	if (gettimeofday(&tv, NULL) != 0) {
 		perror("gettimeofday");
-		return -1;
+		return TEST_ERR;
 	}
 
 	/* initialize dump header to zero */
@@ -253,48 +284,57 @@ writeheader_32(FILE *f)
 	hdr.current_cpu = htodump32(be, current_cpu);
 	hdr.nr_cpus = htodump32(be, nr_cpus);
 
-	if (fseek(f, 0, SEEK_SET) != 0) {
-		perror("seek header");
-		return -1;
-	}
-
-	if (fwrite(&hdr, sizeof hdr, 1, f) != 1) {
-		perror("write header");
-		return -1;
-	}
+	if (write_chunk(f, 0, &hdr, sizeof hdr, "header"))
+		return TEST_ERR;
 
+	pos = DISKDUMP_HEADER_BLOCKS * block_size + sizeof(subhdr);
 	subhdr.phys_base = htodump32(be, phys_base);
 	subhdr.dump_level = htodump32(be, dump_level);
 	subhdr.split = htodump32(be, split);
 	subhdr.start_pfn = htodump32(be, start_pfn);
 	subhdr.end_pfn = htodump32(be, end_pfn);
-	subhdr.offset_vmcoreinfo = htodump64(be, vmcoreinfo.filepos);
-	subhdr.size_vmcoreinfo = htodump32(be, (vmcoreinfo.blob
-						? vmcoreinfo.blob->length
-						: 0));
-	subhdr.offset_note = htodump64(be, notes.filepos);
-	subhdr.size_note = htodump32(be, (notes.blob
-					  ? notes.blob->length
-					  : 0));
-	subhdr.offset_eraseinfo = htodump64(be, eraseinfo.filepos);
-	subhdr.size_eraseinfo = htodump32(be, (eraseinfo.blob
-					       ? eraseinfo.blob->length
-					       : 0));
+	if (vmcoreinfo) {
+		subhdr.offset_vmcoreinfo = htodump64(be, pos);
+		subhdr.size_vmcoreinfo = htodump32(be, vmcoreinfo->length);
+		if (write_chunk(f, pos, vmcoreinfo->data, vmcoreinfo->length,
+				"VMCOREINFO"))
+			return TEST_ERR;
+		pos += vmcoreinfo->length;
+	} else {
+		subhdr.offset_vmcoreinfo = htodump64(be, 0);
+		subhdr.size_vmcoreinfo = htodump32(be, 0);
+	}
+	if (notes) {
+		subhdr.offset_note = htodump64(be, pos);
+		subhdr.size_note = htodump32(be, notes->length);
+		if (write_chunk(f, pos, notes->data, notes->length,
+				"ELF notes"))
+			return TEST_ERR;
+		pos += notes->length;
+	} else {
+		subhdr.offset_note = htodump64(be, 0);
+		subhdr.size_note = htodump32(be, 0);
+	}
+	if (eraseinfo) {
+		subhdr.offset_eraseinfo = htodump64(be, pos);
+		subhdr.size_eraseinfo = htodump32(be, eraseinfo->length);
+		if (write_chunk(f, pos, notes->data, notes->length,
+				"eraseinfo"))
+			return TEST_ERR;
+		pos += eraseinfo->length;
+	} else {
+		subhdr.offset_eraseinfo = htodump64(be, 0);
+		subhdr.size_eraseinfo = htodump32(be, 0);
+	}
 	subhdr.start_pfn_64 = htodump64(be, start_pfn);
 	subhdr.end_pfn_64 = htodump64(be, end_pfn);
 	subhdr.max_mapnr_64 = htodump64(be, max_mapnr);
 
-	if (fseek(f, DISKDUMP_HEADER_BLOCKS * block_size, SEEK_SET) != 0) {
-		perror("seek subheader");
-		return -1;
-	}
+	if (write_chunk(f, DISKDUMP_HEADER_BLOCKS * block_size,
+			&subhdr, sizeof subhdr, "subheader"))
+		return TEST_ERR;
 
-	if (fwrite(&subhdr, sizeof subhdr, 1, f) != 1) {
-		perror("write subheader");
-		return -1;
-	}
-
-	return 0;
+	return TEST_OK;
 }
 
 static int
@@ -303,10 +343,11 @@ writeheader_64(FILE *f)
 	struct timeval tv;
 	struct disk_dump_header_64 hdr;
 	struct kdump_sub_header_64 subhdr;
+	off_t pos;
 
 	if (gettimeofday(&tv, NULL) != 0) {
 		perror("gettimeofday");
-		return -1;
+		return TEST_ERR;
 	}
 
 	/* initialize dump header to zero */
@@ -339,48 +380,57 @@ writeheader_64(FILE *f)
 	hdr.current_cpu = htodump32(be, current_cpu);
 	hdr.nr_cpus = htodump32(be, nr_cpus);
 
-	if (fseek(f, 0, SEEK_SET) != 0) {
-		perror("seek header");
-		return -1;
-	}
-
-	if (fwrite(&hdr, sizeof hdr, 1, f) != 1) {
-		perror("write header");
-		return -1;
-	}
+	if (write_chunk(f, 0, &hdr, sizeof hdr, "header"))
+		return TEST_ERR;
 
+	pos = DISKDUMP_HEADER_BLOCKS * block_size + sizeof(subhdr);
 	subhdr.phys_base = htodump64(be, phys_base);
 	subhdr.dump_level = htodump32(be, dump_level);
 	subhdr.split = htodump32(be, split);
 	subhdr.start_pfn = htodump64(be, start_pfn);
 	subhdr.end_pfn = htodump64(be, end_pfn);
-	subhdr.offset_vmcoreinfo = htodump64(be, vmcoreinfo.filepos);
-	subhdr.size_vmcoreinfo = htodump64(be, (vmcoreinfo.blob
-						? vmcoreinfo.blob->length
-						: 0));
-	subhdr.offset_note = htodump64(be, notes.filepos);
-	subhdr.size_note = htodump64(be, (notes.blob
-					  ? notes.blob->length
-					  : 0));
-	subhdr.offset_eraseinfo = htodump64(be, eraseinfo.filepos);
-	subhdr.size_eraseinfo = htodump64(be, (eraseinfo.blob
-					       ? eraseinfo.blob->length
-					       : 0));
+	if (vmcoreinfo) {
+		subhdr.offset_vmcoreinfo = htodump64(be, pos);
+		subhdr.size_vmcoreinfo = htodump64(be, vmcoreinfo->length);
+		if (write_chunk(f, pos, vmcoreinfo->data, vmcoreinfo->length,
+				"VMCOREINFO"))
+			return TEST_ERR;
+		pos += vmcoreinfo->length;
+	} else {
+		subhdr.offset_vmcoreinfo = htodump64(be, 0);
+		subhdr.size_vmcoreinfo = htodump64(be, 0);
+	}
+	if (notes) {
+		subhdr.offset_note = htodump64(be, pos);
+		subhdr.size_note = htodump64(be, notes->length);
+		if (write_chunk(f, pos, notes->data, notes->length,
+				"ELF notes"))
+			return TEST_ERR;
+		pos += notes->length;
+	} else {
+		subhdr.offset_note = htodump64(be, 0);
+		subhdr.size_note = htodump64(be, 0);
+	}
+	if (eraseinfo) {
+		subhdr.offset_eraseinfo = htodump64(be, pos);
+		subhdr.size_eraseinfo = htodump64(be, eraseinfo->length);
+		if (write_chunk(f, pos, notes->data, notes->length,
+				"eraseinfo"))
+			return TEST_ERR;
+		pos += eraseinfo->length;
+	} else {
+		subhdr.offset_eraseinfo = htodump64(be, 0);
+		subhdr.size_eraseinfo = htodump64(be, 0);
+	}
 	subhdr.start_pfn_64 = htodump64(be, start_pfn);
 	subhdr.end_pfn_64 = htodump64(be, end_pfn);
 	subhdr.max_mapnr_64 = htodump64(be, max_mapnr);
 
-	if (fseek(f, DISKDUMP_HEADER_BLOCKS * block_size, SEEK_SET) != 0) {
-		perror("seek subheader");
-		return -1;
-	}
+	if (write_chunk(f, DISKDUMP_HEADER_BLOCKS * block_size,
+			&subhdr, sizeof subhdr, "subheader"))
+		return TEST_ERR;
 
-	if (fwrite(&subhdr, sizeof subhdr, 1, f) != 1) {
-		perror("write subheader");
-		return -1;
-	}
-
-	return 0;
+	return TEST_OK;
 }
 
 static int
@@ -403,24 +453,11 @@ parseheader(struct page_data *pg, char *p)
 
 	pgkdump->flags = 0;
 	pgkdump->compress = compress_auto;
-	pgkdump->skip = 0;
 
 	p = endp;
 	while (*p && isspace(*p))
 		++p;
 
-	if (!strncmp(p, "skip=", 5)) {
-		p += 5;
-		pgkdump->skip = strtoul(p, &endp, 0);
-		if (*endp && !isspace(*endp)) {
-			fprintf(stderr, "Invalid skip: %s\n", p);
-			return TEST_FAIL;
-		}
-		p = endp;
-		while (*p && isspace(*p))
-			++p;
-	}
-
 	if (!*p)
 		return TEST_OK;
 
@@ -437,6 +474,9 @@ parseheader(struct page_data *pg, char *p)
 		pgkdump->compress = compress_yes;
 	} else if (!strcmp(p, "exclude")) {
 		pgkdump->compress = compress_exclude;
+	} else if (!strcmp(p, "zstd")) {
+		pgkdump->flags |= DUMP_DH_COMPRESSED_ZSTD;
+		pgkdump->compress = compress_yes;
 	} else {
 		pgkdump->flags = strtoul(p, &endp, 0);
 		if (*endp) {
@@ -448,7 +488,7 @@ parseheader(struct page_data *pg, char *p)
 	return TEST_OK;
 }
 
-#if USE_ZLIB || USE_LZO || USE_SNAPPY
+#if USE_ZLIB || USE_LZO || USE_SNAPPY || USE_ZSTD
 static size_t
 enlarge_cbuf(struct page_data_kdump *pgkdump, size_t newsz)
 {
@@ -524,6 +564,27 @@ do_snappy(struct page_data *pg)
 }
 #endif
 
+#if USE_ZSTD
+static size_t
+do_zstd(struct page_data *pg)
+{
+	struct page_data_kdump *pgkdump = pg->priv;
+	size_t clen;
+
+	clen = ZSTD_compressBound(pg->len);
+	if (clen > pgkdump->cbufsz &&
+	    !(clen = enlarge_cbuf(pgkdump, clen)))
+		return clen;
+
+	clen = ZSTD_compress(pgkdump->cbuf, clen, pg->buf, pg->len, 1);
+	if (ZSTD_isError(clen)) {
+		fprintf(stderr, "zstd compression failed\n");
+		clen = 0;
+	}
+	return clen;
+}
+#endif
+
 static size_t
 compresspage(struct page_data *pg, uint32_t *pflags)
 {
@@ -538,6 +599,9 @@ compresspage(struct page_data *pg, uint32_t *pflags)
 		case COMPRESS_SNAPPY:
 			*pflags |= DUMP_DH_COMPRESSED_SNAPPY;
 			break;
+		case COMPRESS_ZSTD:
+			*pflags |= DUMP_DH_COMPRESSED_ZSTD;
+			break;
 		}
 
 #if USE_ZLIB
@@ -552,6 +616,10 @@ compresspage(struct page_data *pg, uint32_t *pflags)
 	if (*pflags & DUMP_DH_COMPRESSED_SNAPPY)
 		return do_snappy(pg);
 #endif
+#if USE_ZSTD
+	if (*pflags & DUMP_DH_COMPRESSED_ZSTD)
+		return do_zstd(pg);
+#endif
 
 	fprintf(stderr, "Unsupported compression flags: %lu\n",
 		(unsigned long) *pflags);
@@ -611,12 +679,17 @@ writepage(struct page_data *pg)
 	struct page_desc pd;
 	unsigned long pdidx;
 	unsigned char *buf;
+	unsigned long pfn;
 	size_t buflen;
 	uint32_t flags;
 
 	if (pgkdump->compress == compress_exclude)
 		return TEST_OK;
 
+	pfn = pgkdump->addr / block_size;
+	if (split && (start_pfn > pfn || pfn >= end_pfn))
+		return TEST_OK;
+
 	flags = pgkdump->flags;
 
 	if (pg->len &&
@@ -640,36 +713,19 @@ writepage(struct page_data *pg)
 		buf = pg->buf;
 	}
 	pd.offset = htodump64(be, dataoff);
-	pd.size = htodump32(be, buflen + pgkdump->skip);
+	pd.size = htodump32(be, buflen);
 	pd.flags = htodump32(be, flags);
 	pd.page_flags = htodump64(be, 0);
 
-	pdidx = bitmap_index(bitmap2, pgkdump->addr / block_size);
-	if (fseek(pgkdump->f, pdoff + pdidx * sizeof pd, SEEK_SET) != 0) {
-		perror("seek page desc");
+	pdidx = bitmap_index(bitmap2, pfn) - start_pdidx;
+	if (write_chunk(pgkdump->f, pdoff + pdidx * sizeof pd,
+			&pd, sizeof pd, "page desc"))
 		return TEST_ERR;
-	}
-	if (fwrite(&pd, sizeof pd, 1, pgkdump->f) != 1) {
-		perror("write page desc");
-		return TEST_ERR;
-	}
 
-	if (fseek(pgkdump->f, dataoff, SEEK_SET) != 0) {
-		perror("seek page data");
+	if (write_chunk(pgkdump->f, dataoff, buf, buflen, "page data"))
 		return TEST_ERR;
-	}
-	if (fwrite(buf, 1, buflen, pgkdump->f) != buflen) {
-		perror("write page data");
-		return TEST_ERR;
-	}
 	dataoff += buflen;
 
-	if (pgkdump->skip &&
-	    fseek(pgkdump->f, pgkdump->skip, SEEK_CUR) != 0) {
-		perror("skip page data");
-		return TEST_ERR;
-	}
-
 	return TEST_OK;
 }
 
@@ -682,7 +738,7 @@ writedata(FILE *f)
 	int rc;
 
 	if (!data_file)
-		return TEST_OK;
+		return writeheader(f);
 
 	pgkdump.f = f;
 	pgkdump.addr = 0;
@@ -737,24 +793,25 @@ writedata(FILE *f)
 	if (rc != TEST_OK)
 		goto out_bitmap2;
 
-	if (fseek(f, (1 + sub_hdr_size) * block_size, SEEK_SET) != 0) {
-		perror("seek bitmap");
+	rc = writeheader(f);
+	if (rc != TEST_OK)
+		goto out_bitmap2;
+
+	if (write_chunk(f, (1 + sub_hdr_size) * block_size,
+			bitmap1, block_size * bmp_blocks1, "1st bitmap")) {
 		rc = TEST_ERR;
 		goto out_bitmap2;
 	}
-	if (fwrite(bitmap1, block_size, bmp_blocks1, f) != bmp_blocks1) {
-		perror("write 1st bitmap");
-		rc = TEST_ERR;
-		goto out_bitmap2;
-	}
-	if (fwrite(bitmap2, block_size, bmp_blocks1, f) != bmp_blocks1) {
-		perror("write 2nd bitmap");
+	if (write_chunk(f, (1 + sub_hdr_size + bmp_blocks1) * block_size,
+			bitmap2, block_size * bmp_blocks1, "2nd bitmap")) {
 		rc = TEST_ERR;
 		goto out_bitmap2;
 	}
 
 	printf("Creating page data\n");
 
+	if (split)
+		start_pdidx = bitmap_index(bitmap2, start_pfn);
 	dataoff += block_size - (dataoff - 1) % block_size - 1;
 
 	pg.endian = be;
@@ -783,13 +840,41 @@ writedump(FILE *f)
 {
 	int rc;
 
+	if (flattened) {
+		struct makedumpfile_header hdr = {
+			.signature = MDF_SIGNATURE,
+			.type = htobe64(flattened_type),
+			.version = htobe64(flattened_version),
+		};
+		size_t remain;
+
+		if (fwrite(&hdr, sizeof hdr, 1, f) != 1) {
+			perror("flattened header");
+			return TEST_ERR;
+		}
+		remain = MDF_HEADER_SIZE - sizeof hdr;
+		while (remain--) {
+			if (putc(0, f) != 0) {
+				perror("flattened header padding");
+				return TEST_ERR;
+			}
+		}
+	}
+
 	rc = writedata(f);
 	if (rc != 0)
 		return rc;
 
-	rc = writeheader(f);
-	if (rc != 0)
-		return rc;
+	if (flattened) {
+		struct makedumpfile_data_header hdr = {
+			.offset = htobe64(MDF_OFFSET_END_FLAG),
+			.buf_size = htobe64(MDF_OFFSET_END_FLAG),
+		};
+		if (fwrite(&hdr, sizeof hdr, 1, f) != 1) {
+			perror("end segment header");
+			return TEST_ERR;
+		}
+	}
 
 	return 0;
 }
@@ -861,20 +946,20 @@ main(int argc, char **argv)
 		return rc;
 
 	if (vmcoreinfo_file) {
-		vmcoreinfo.blob = slurp(vmcoreinfo_file);
-		if (vmcoreinfo.blob == NULL)
+		vmcoreinfo = slurp(vmcoreinfo_file);
+		if (vmcoreinfo == NULL)
 			return TEST_ERR;
 	}
 
 	if (note_file) {
-		notes.blob = slurp(note_file);
-		if (notes.blob == NULL)
+		notes = slurp(note_file);
+		if (notes == NULL)
 			return TEST_ERR;
 	}
 
 	if (eraseinfo_file) {
-		eraseinfo.blob = slurp(eraseinfo_file);
-		if (eraseinfo.blob == NULL)
+		eraseinfo = slurp(eraseinfo_file);
+		if (eraseinfo == NULL)
 			return TEST_ERR;
 	}
 
