@@ -26,6 +26,8 @@
    not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "testutil.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <time.h>
@@ -36,8 +38,6 @@
 #include <pthread.h>
 #include <libkdumpfile/kdumpfile.h>
 
-#include "testutil.h"
-
 #define DEFITER		1000
 #define DEFTHREADS	1
 
@@ -152,7 +152,7 @@ run_threads_fd(int fd, unsigned long nthreads, unsigned long cache_size)
 		return TEST_ERR;
 	}
 
-	res = kdump_set_number_attr(ctx, KDUMP_ATTR_FILE_FD, fd);
+	res = kdump_open_fd(ctx, fd);
 	if (res != KDUMP_OK) {
 		fprintf(stderr, "Cannot open dump: %s\n", kdump_get_err(ctx));
 		rc = TEST_ERR;
