@@ -28,12 +28,12 @@
 
 #define _GNU_SOURCE
 
+#include "testutil.h"
+
 #include <stdio.h>
 
 #include <libkdumpfile/addrxlat.h>
 
-#include "testutil.h"
-
 static int
 setup_pgt(addrxlat_ctx_t *ctx, addrxlat_sys_t *sys)
 {
@@ -70,8 +70,20 @@ setup_pgt(addrxlat_ctx_t *ctx, addrxlat_sys_t *sys)
 	return TEST_OK;
 }
 
+static unsigned long
+machphysaddr_caps(const addrxlat_cb_t *cb)
+{
+	return ADDRXLAT_CAPS(ADDRXLAT_MACHPHYSADDR);
+}
+
+static unsigned long
+kvaddr_caps(const addrxlat_cb_t *cb)
+{
+	return ADDRXLAT_CAPS(ADDRXLAT_KVADDR);
+}
+
 static addrxlat_status
-mygetpage(void *data, addrxlat_buffer_t *buf)
+mygetpage(const addrxlat_cb_t *cb, addrxlat_buffer_t *buf)
 {
 	fputs("read callback called?!\n", stderr);
 	return ADDRXLAT_OK;
@@ -86,12 +98,9 @@ nullop(void *data, const addrxlat_fulladdr_t *paddr)
 int
 main(int argc, char **argv)
 {
-	static addrxlat_cb_t cb = {
-		.get_page = mygetpage,
-		.read_caps = ADDRXLAT_CAPS(ADDRXLAT_MACHPHYSADDR)
-	};
-
 	addrxlat_ctx_t *ctx;
+	const addrxlat_cb_t *def_cb;
+	addrxlat_cb_t *cb;
 	addrxlat_sys_t *sys;
 	addrxlat_fulladdr_t faddr;
 	addrxlat_op_ctl_t opctl;
@@ -104,7 +113,14 @@ main(int argc, char **argv)
 		return TEST_ERR;
 	}
 
-	addrxlat_ctx_set_cb(ctx, &cb);
+	def_cb = addrxlat_ctx_get_cb(ctx);
+	cb = addrxlat_ctx_add_cb(ctx);
+	if (!cb) {
+		fputs("Cannot allocate translation callbacks", stderr);
+		return TEST_ERR;
+	}
+	cb->get_page = mygetpage;
+	cb->read_caps = machphysaddr_caps;
 
 	sys = addrxlat_sys_new();
 	if (!sys) {
@@ -140,8 +156,7 @@ main(int argc, char **argv)
 		printf("OK (%s)\n", addrxlat_ctx_get_err(ctx));
 
 	fputs("Callback with no capabilities: ", stdout);
-	cb.read_caps = 0;
-	addrxlat_ctx_set_cb(ctx, &cb);
+	cb->read_caps = def_cb->read_caps;
 	status = addrxlat_op(&opctl, &faddr);
 	if (status == ADDRXLAT_OK) {
 		puts("FAIL");
@@ -156,8 +171,9 @@ main(int argc, char **argv)
 		printf("OK (%s)\n", addrxlat_ctx_get_err(ctx));
 
 	fputs("Missing callback: ", stdout);
-	cb.get_page = NULL;
-	addrxlat_ctx_set_cb(ctx, &cb);
+	cb->priv = def_cb->priv;
+	cb->get_page = def_cb->get_page;
+	cb->read_caps = kvaddr_caps;
 	status = addrxlat_op(&opctl, &faddr);
 	if (status == ADDRXLAT_OK) {
 		puts("FAIL");
