@@ -26,19 +26,19 @@
    not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "testutil.h"
+
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "testutil.h"
-
 #define ARRAY_SEPARATORS  " \t"
 
 static char
 unescape_hex(const char **pp)
 {
-	char n, ret;
+	signed char n, ret;
 
 	ret = unhex(**pp);
 	if (ret < 0) {
@@ -60,7 +60,7 @@ unescape_hex(const char **pp)
 static char
 unescape_oct(const char **pp)
 {
-	char n, ret;
+	signed char n, ret;
 
 	ret = unoct(**pp);
 	++(*pp);
@@ -152,6 +152,30 @@ set_param_string(const struct param *param, const char *val)
 	return TEST_OK;
 }
 
+static int
+set_param_yesno(const struct param *param, const char *val)
+{
+	if (!strcasecmp(val, "yes") ||
+	    !strcasecmp(val, "true")) {
+		*param->yesno = true;
+	} else if (!strcasecmp(val, "no") ||
+		   !strcasecmp(val, "false")) {
+		*param->yesno = false;
+	} else {
+		unsigned long long num;
+		char *endp;
+
+		num = strtoull(val, &endp, 0);
+		if (!*val || *endp) {
+			fprintf(stderr, "Invalid yes/no value: %s\n", val);
+			return TEST_FAIL;
+		}
+		*param->yesno = !!num;
+	}
+
+	return TEST_OK;
+}
+
 static int
 set_param_number(const struct param *param, const char *val)
 {
@@ -219,7 +243,6 @@ set_param_blob(const struct param *param, const char *val)
 	unsigned i, n;
 	const char *p;
 	unsigned char *valp;
-	char *endp;
 
 	p = strpbrk(val, ARRAY_SEPARATORS);
 	n = p ? p - val : strlen(val);
@@ -249,7 +272,7 @@ set_param_blob(const struct param *param, const char *val)
 			}
 		}
 		if (!isok) {
-			fprintf(stderr, "Invalid hex digit at %d: '%c'\n",
+			fprintf(stderr, "Invalid hex digit at %zu: '%c'\n",
 				p - val, *p);
 			free(blob);
 			return TEST_FAIL;
@@ -263,6 +286,66 @@ set_param_blob(const struct param *param, const char *val)
 	return TEST_OK;
 }
 
+static int
+set_param_fulladdr(const struct param *param, const char *val)
+{
+	const char *p = val;
+	const char *q;
+	char *endp;
+
+	if (isdigit(*p)) {
+		param->fulladdr->as = strtoul(p, &endp, 0);
+		if (!*p || *endp)
+			goto err;
+		p = endp;
+	} else {
+		q = p;
+		while (isalnum(*q))
+			++q;
+
+		switch (q - p) {
+		case 6:
+			if (!strncasecmp(p, "KVADDR", 6))
+				param->fulladdr->as = ADDRXLAT_KVADDR;
+			else
+				goto err;
+			break;
+
+		case 9:
+			if (!strncasecmp(p, "KPHYSADDR", 9))
+				param->fulladdr->as = ADDRXLAT_KPHYSADDR;
+			else
+				goto err;
+			break;
+
+		case 12:
+			if (!strncasecmp(p, "MACHPHYSADDR", 12))
+				param->fulladdr->as = ADDRXLAT_MACHPHYSADDR;
+			else
+				goto err;
+			break;
+
+		default:
+			goto err;
+		}
+		p = q;
+	}
+
+	if (*p != ':')
+		goto err;
+	++p;
+
+	param->fulladdr->addr = strtoull(p, &endp, 0);
+	if (!*p || *endp)
+		goto err;
+
+	return TEST_OK;
+
+ err:
+	fprintf(stderr, "Invalid full address: %s\n", val);
+	return TEST_FAIL;
+}
+
 int
 set_param(const struct param *p, const char *val)
 {
@@ -270,6 +353,9 @@ set_param(const struct param *p, const char *val)
 	case param_string:
 		return set_param_string(p, val);
 
+	case param_yesno:
+		return set_param_yesno(p, val);
+
 	case param_number:
 		return set_param_number(p, val);
 
@@ -278,6 +364,9 @@ set_param(const struct param *p, const char *val)
 
 	case param_blob:
 		return set_param_blob(p, val);
+
+	case param_fulladdr:
+		return set_param_fulladdr(p, val);
 	}
 
 	fprintf(stderr, "INTERNAL ERROR: Invalid param type: %d\n",
