@@ -28,6 +28,8 @@
 
 #define _GNU_SOURCE
 
+#include "testutil.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -36,8 +38,6 @@
 #include <unistd.h>
 #include <libkdumpfile/addrxlat.h>
 
-#include "testutil.h"
-
 static char *data_file;
 static char *cfg_file;
 
@@ -120,18 +120,6 @@ static const kw_pair_t map_names[] = {
 	{NULL}
 };
 
-static const kw_pair_t pte_formats[] = {
-	{ "none", ADDRXLAT_PTE_NONE },
-	{ "pfn32", ADDRXLAT_PTE_PFN32 },
-	{ "pfn64", ADDRXLAT_PTE_PFN64 },
-	{ "ia32", ADDRXLAT_PTE_IA32 },
-	{ "ia32_pae", ADDRXLAT_PTE_IA32_PAE },
-	{ "x86_64", ADDRXLAT_PTE_X86_64 },
-	{ "s390x", ADDRXLAT_PTE_S390X },
-	{ "ppc64_linux_rpn30", ADDRXLAT_PTE_PPC64_LINUX_RPN30 },
-	{NULL}
-};
-
 enum param_index {
 	/* Generic */
 	param_kind,
@@ -251,11 +239,12 @@ parse_kind(const char *spec, addrxlat_kind_t *kind)
 static int
 parse_pte_format(const char *spec, addrxlat_pte_format_t *fmt)
 {
-	long i = match_keyword_verb("PTE format", spec, strlen(spec),
-				    pte_formats);
-	if (i == NOTFOUND)
+	addrxlat_pte_format_t res = addrxlat_pte_format(spec);
+	if (res == ADDRXLAT_PTE_INVALID) {
+		fprintf(stderr, "Unknown PTE format: %s\n", spec);
 		return TEST_ERR;
-	*fmt = i;
+	}
+	*fmt = res;
 	return TEST_OK;
 }
 
@@ -777,12 +766,17 @@ find_entry(addrxlat_addr_t addr, size_t sz)
 	return NULL;
 }
 
-static addrxlat_status
-get_page(void *data, addrxlat_buffer_t *buf)
+static unsigned long
+read_caps(const addrxlat_cb_t *cb)
 {
-	struct cbdata *cbd = data;
+	return ADDRXLAT_CAPS(ADDRXLAT_MACHPHYSADDR);
+}
+
+static addrxlat_status
+get_page(const addrxlat_cb_t *cb, addrxlat_buffer_t *buf)
+{
+	struct cbdata *cbd = cb->priv;
 	struct entry *ent;
-	uint64_t *p;
 
 	if (buf->addr.as != ADDRXLAT_MACHPHYSADDR)
 		return addrxlat_ctx_err(cbd->ctx, ADDRXLAT_ERR_INVALID,
@@ -904,11 +898,7 @@ int main(int argc, char *argv[])
 	char *paramfn = NULL;
 	FILE *param, *cfg;
 	struct cbdata data;
-	addrxlat_cb_t cb = {
-		.data = &data,
-		.get_page = get_page,
-		.read_caps = ADDRXLAT_CAPS(ADDRXLAT_MACHPHYSADDR)
-	};
+	addrxlat_cb_t *cb;
 	int c;
 	int i;
 	int rc;
@@ -951,7 +941,14 @@ int main(int argc, char *argv[])
 		perror("Cannot allocate addrxlat");
 		return TEST_ERR;
 	}
-	addrxlat_ctx_set_cb(data.ctx, &cb);
+	cb = addrxlat_ctx_add_cb(data.ctx);
+	if (!cb) {
+		perror("Cannot allocate addrxlat callbacks");
+		return TEST_ERR;
+	}
+	cb->priv = &data;
+	cb->get_page = get_page;
+	cb->read_caps = read_caps;
 
 	cfg = fopen(cfg_file, "r");
 	if (!cfg) {
