@@ -29,10 +29,15 @@
 #ifndef _TESTUTIL_H
 #define _TESTUTIL_H 1
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <endian.h>
 
+#include <libkdumpfile/addrxlat.h>
+
 #define TEST_OK     0
 #define TEST_FAIL   1
 #define TEST_SKIP  77
@@ -72,7 +77,7 @@ htodump64(endian_t endian, uint64_t x)
 }
 
 /* Hex/oct */
-static inline char
+static inline signed char
 unhex(char digit)
 {
 	if (digit >= '0' && digit <= '9')
@@ -87,7 +92,7 @@ unhex(char digit)
 	return -1;
 }
 
-static inline char
+static inline signed char
 unoct(char digit)
 {
 	if (digit >= '0' && digit <= '7')
@@ -114,15 +119,19 @@ struct param {
 	const char *key;
 	enum {
 		param_string,
+		param_yesno,
 		param_number,
 		param_number_array,
 		param_blob,
+		param_fulladdr,
 	} type;
 	union {
 		char **string;
+		bool *yesno;
 		unsigned long long *number;
 		struct number_array *number_array;
 		struct blob **blob;
+		addrxlat_fulladdr_t *fulladdr;
 	};
 };
 
@@ -133,10 +142,14 @@ struct params {
 
 #define PARAM_STRING(key, val) \
 	{ (key), param_string, { .string = &(val) } }
+#define PARAM_YESNO(key, val) \
+	{ (key), param_yesno, { .yesno = &(val) } }
 #define PARAM_NUMBER(key, val) \
 	{ (key), param_number, { .number = &(val) } }
 #define PARAM_NUMBER_ARRAY(key, val) \
 	{ (key), param_number_array, { .number_array = &(val) } }
+#define PARAM_FULLADDR(key, val) \
+	{ (key), param_fulladdr, { .fulladdr = &(val) } }
 
 int parse_key_val(char *line, char **key, char **val);
 
