@@ -26,12 +26,12 @@
    not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "testutil.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <libkdumpfile/kdumpfile.h>
 
-#include "testutil.h"
-
 #define xstr(s)	#s
 #define str(s)	xstr(s)
 
@@ -81,7 +81,7 @@ set_vmcoreinfo_value(kdump_ctx_t *ctx, const char *val)
 
 	blob = kdump_blob_new_dup(val, strlen(val));
 	if (!blob) {
-		fprintf(stderr, "#%d: Cannot allocate VMCOREINFO blob.\n");
+		fprintf(stderr, "Cannot allocate VMCOREINFO blob.\n");
 		return TEST_ERR;
 	}
 	attr.type = KDUMP_BLOB;
