@@ -26,14 +26,18 @@
    not, see <http://www.gnu.org/licenses/>.
 */
 
+#define _GNU_SOURCE
+
+#include "testutil.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <endian.h>
 #include <ctype.h>
-#include <libkdumpfile/addrxlat.h>
+#include <limits.h>
 
-#include "testutil.h"
+#include <libkdumpfile/addrxlat.h>
 
 struct cbdata {
 	addrxlat_ctx_t *ctx;
@@ -64,12 +68,17 @@ find_entry(addrxlat_addr_t addr, size_t sz)
 	return NULL;
 }
 
-static addrxlat_status
-get_page(void *data, addrxlat_buffer_t *buf)
+static unsigned long
+read_caps(const addrxlat_cb_t *cb)
 {
-	struct cbdata *cbd = data;
+	return ADDRXLAT_CAPS(entry_as);
+}
+
+static addrxlat_status
+get_page(const addrxlat_cb_t *cb, addrxlat_buffer_t *buf)
+{
+	struct cbdata *cbd = cb->priv;
 	struct entry *ent;
-	uint32_t *p;
 
 	if (buf->addr.as != entry_as)
 		return addrxlat_ctx_err(cbd->ctx, ADDRXLAT_ERR_INVALID,
@@ -107,16 +116,28 @@ add_entry(addrxlat_addr_t addr, void *buf, size_t sz)
 	return TEST_OK;
 }
 
+#define SYM_ARGC_MAX	2
+
 struct symdata {
 	struct symdata *next;
-	addrxlat_sym_t ss;
+	const char *args[SYM_ARGC_MAX];
 	addrxlat_addr_t val;
 };
 
-static struct symdata *symdata;
+struct symstore {
+	struct symdata **list;
+	struct symdata data;
+};
+
+static struct symdata *symdata_reg;
+static struct symdata *symdata_value;
+static struct symdata *symdata_sizeof;
+static struct symdata *symdata_offsetof;
+static struct symdata *symdata_number;
 
 static int
-add_symdata(const addrxlat_sym_t *ss, addrxlat_addr_t val)
+add_symdata(struct symdata **list, const char *const *args,
+	    addrxlat_addr_t val)
 {
 	struct symdata *sd = malloc(sizeof(*sd));
 	if (!sd) {
@@ -124,59 +145,98 @@ add_symdata(const addrxlat_sym_t *ss, addrxlat_addr_t val)
 		return TEST_ERR;
 	}
 
-	sd->ss = *ss;
+	memcpy(&sd->args, args, sizeof sd->args);
 	sd->val = val;
-	sd->next = symdata;
-	symdata = sd;
+	sd->next = *list;
+	*list = sd;
 	return TEST_OK;
 }
 
 static addrxlat_status
-get_symdata(void *data, addrxlat_sym_t *sym)
+get_symdata_offsetof(const addrxlat_cb_t *cb, const char *obj,
+		     const char *elem, addrxlat_addr_t *val)
 {
 	struct symdata *sd;
-	for (sd = symdata; sd; sd = sd->next) {
-		if (sd->ss.type != sym->type)
-			continue;
-
-		switch (sd->ss.type) {
-		case ADDRXLAT_SYM_REG:
-		case ADDRXLAT_SYM_VALUE:
-		case ADDRXLAT_SYM_SIZEOF:
-		case ADDRXLAT_SYM_NUMBER:
-			if (sd->ss.args[0] &&
-			    !strcmp(sd->ss.args[0], sym->args[0])) {
-				sym->val = sd->val;
-				return ADDRXLAT_OK;
-			}
-			break;
-
-		case ADDRXLAT_SYM_OFFSETOF:
-			if (sd->ss.args[0] && sd->ss.args[1] &&
-			    !strcmp(sd->ss.args[0], sym->args[0]) &&
-			    !strcmp(sd->ss.args[1], sym->args[1])) {
-				sym->val = sd->val;
-				return ADDRXLAT_OK;
-			}
+	for (sd = symdata_offsetof; sd; sd = sd->next)
+		if (sd->args[0] && sd->args[1] &&
+		    !strcmp(sd->args[0], obj) &&
+		    !strcmp(sd->args[1], elem)) {
+			*val = sd->val;
+			return ADDRXLAT_OK;
 		}
-	}
 
 	return ADDRXLAT_ERR_NODATA;
 }
 
-static unsigned long long ostype;
-static unsigned long long osver;
+static addrxlat_status
+get_symdata_list(const addrxlat_cb_t *cb, const char *name,
+		 addrxlat_addr_t *val, struct symdata *sd)
+{
+	for (; sd; sd = sd->next)
+		if (sd->args[0] &&
+		    !strcmp(sd->args[0], name)) {
+			*val = sd->val;
+			return ADDRXLAT_OK;
+		}
+
+	return ADDRXLAT_ERR_NODATA;
+}
+
+static addrxlat_status
+get_symdata_reg(const addrxlat_cb_t *cb, const char *name,
+		addrxlat_addr_t *val)
+{
+	return get_symdata_list(cb, name, val, symdata_reg);
+}
+
+static addrxlat_status
+get_symdata_value(const addrxlat_cb_t *cb, const char *name,
+		  addrxlat_addr_t *val)
+{
+	return get_symdata_list(cb, name, val, symdata_value);
+}
+
+static addrxlat_status
+get_symdata_sizeof(const addrxlat_cb_t *cb, const char *name,
+		   addrxlat_addr_t *val)
+{
+	return get_symdata_list(cb, name, val, symdata_sizeof);
+}
+
+static addrxlat_status
+get_symdata_number(const addrxlat_cb_t *cb, const char *name,
+		   addrxlat_addr_t *val)
+{
+	return get_symdata_list(cb, name, val, symdata_number);
+}
+
 static char *arch;
-static char *opts;
+static char *ostype;
+static unsigned long long osver;
+static unsigned long long phys_bits;
+static unsigned long long virt_bits;
+static unsigned long long page_shift;
+static unsigned long long phys_base;
+static addrxlat_fulladdr_t rootpgt;
+static unsigned long long xen_p2m_mfn;
+static bool xen_xlat;
 
 static char *sym_file;
 static char *data_file;
 
 static const struct param param_array[] = {
-	PARAM_NUMBER("ostype", ostype),
-	PARAM_NUMBER("osver", osver),
+	/* addrxlat options */
 	PARAM_STRING("arch", arch),
-	PARAM_STRING("opts", opts),
+	PARAM_STRING("ostype", ostype),
+	PARAM_NUMBER("osver", osver),
+	PARAM_NUMBER("phys_bits", phys_bits),
+	PARAM_NUMBER("virt_bits", virt_bits),
+	PARAM_NUMBER("page_shift", page_shift),
+	PARAM_NUMBER("phys_base", phys_base),
+	PARAM_FULLADDR("rootpgt", rootpgt),
+	PARAM_NUMBER("xen_p2m_mfn", xen_p2m_mfn),
+	PARAM_YESNO("xen_xlat", xen_xlat),
+
 	PARAM_NUMBER("data_as", entry_as),
 
 	PARAM_STRING("SYM", sym_file),
@@ -188,43 +248,87 @@ static const struct params params = {
 	param_array
 };
 
-static void
-print_addrspace(addrxlat_addrspace_t as)
+static void clear_params(void)
 {
-	switch (as) {
-	case ADDRXLAT_KPHYSADDR:
-		fputs("KPHYSADDR", stdout);
-		break;
+	arch = NULL;
+	ostype = NULL;
+	osver = ULLONG_MAX;
+	phys_bits = ULLONG_MAX;
+	virt_bits = ULLONG_MAX;
+	page_shift = ULLONG_MAX;
+	phys_base = ULLONG_MAX;
+	rootpgt.as = ADDRXLAT_NOADDR;
+	xen_p2m_mfn = ULLONG_MAX;
+	xen_xlat = false;
+}
 
-	case ADDRXLAT_MACHPHYSADDR:
-		fputs("MACHPHYSADDR", stdout);
-		break;
+static unsigned make_opts(addrxlat_opt_t *opts)
+{
+	addrxlat_opt_t *opt = opts;
 
-	case ADDRXLAT_KVADDR:
-		fputs("KVADDR", stdout);
-		break;
-
-	case ADDRXLAT_NOADDR:
-		fputs("NOADDR", stdout);
-		break;
-
-	default:
-		printf("<addrspace %ld>", (long) as);
+	if (arch) {
+		addrxlat_opt_arch(opt, arch);
+		++opt;
 	}
+
+	if (ostype) {
+		addrxlat_opt_os_type(opt, ostype);
+		++opt;
+	}
+
+	if (osver != ULLONG_MAX) {
+		addrxlat_opt_version_code(opt, osver);
+		++opt;
+	}
+
+	if (phys_bits != ULLONG_MAX) {
+		addrxlat_opt_phys_bits(opt, phys_bits);
+		++opt;
+	}
+
+	if (virt_bits != ULLONG_MAX) {
+		addrxlat_opt_virt_bits(opt, virt_bits);
+		++opt;
+	}
+
+	if (page_shift != ULLONG_MAX) {
+		addrxlat_opt_page_shift(opt, page_shift);
+		++opt;
+	}
+
+	if (phys_base != ULLONG_MAX) {
+		addrxlat_opt_phys_base(opt, phys_base);
+		++opt;
+	}
+
+	if (rootpgt.as != ADDRXLAT_NOADDR) {
+		addrxlat_opt_rootpgt(opt, &rootpgt);
+		++opt;
+	}
+
+	if (xen_p2m_mfn != ULLONG_MAX) {
+		addrxlat_opt_xen_p2m_mfn(opt, xen_p2m_mfn);
+		++opt;
+	}
+
+	if (xen_xlat) {
+		addrxlat_opt_xen_xlat(opt, 1);
+		++opt;
+	}
+
+	return opt - opts;
 }
 
 static void
 print_target_as(const addrxlat_meth_t *meth)
 {
-	fputs("  target_as=", stdout);
-	print_addrspace(meth->target_as);
-	putchar('\n');
+	printf("  target_as=%s\n", addrxlat_addrspace_name(meth->target_as));
 }
 
 static void
 print_fulladdr(const addrxlat_fulladdr_t *addr)
 {
-	print_addrspace(addr->as);
+	fputs(addrxlat_addrspace_name(addr->as), stdout);
 	if (addr->as != ADDRXLAT_NOADDR)
 		printf(":0x%"ADDRXLAT_PRIxADDR, addr->addr);
 }
@@ -241,19 +345,8 @@ print_linear(const addrxlat_meth_t *meth)
 static void
 print_pgt(const addrxlat_meth_t *meth)
 {
-	static const char *pte_formats[] = {
-		[ADDRXLAT_PTE_NONE] = "none",
-		[ADDRXLAT_PTE_PFN32] = "pfn32",
-		[ADDRXLAT_PTE_PFN64] = "pfn64",
-		[ADDRXLAT_PTE_AARCH64] = "aarch64",
-		[ADDRXLAT_PTE_IA32] = "ia32",
-		[ADDRXLAT_PTE_IA32_PAE] = "ia32_pae",
-		[ADDRXLAT_PTE_X86_64] = "x86_64",
-		[ADDRXLAT_PTE_S390X] = "s390x",
-		[ADDRXLAT_PTE_PPC64_LINUX_RPN30] = "ppc64_linux_rpn30",
-	};
-
 	const addrxlat_paging_form_t *pf = &meth->param.pgt.pf;
+	const char *pte_fmt;
 	unsigned i;
 
 	puts("PGT");
@@ -264,9 +357,9 @@ print_pgt(const addrxlat_meth_t *meth)
 	printf("  pte_mask=0x%"ADDRXLAT_PRIxPTE"\n",
 	       meth->param.pgt.pte_mask);
 	fputs("  pte_format=", stdout);
-	if (pf->pte_format < ARRAY_SIZE(pte_formats) &&
-	    pte_formats[pf->pte_format])
-		printf("%s", pte_formats[pf->pte_format]);
+	pte_fmt = addrxlat_pte_format_name(pf->pte_format);
+	if (pte_fmt)
+		printf("%s", pte_fmt);
 	else
 		printf("%u", pf->pte_format);
 	printf("\n  fields=");
@@ -394,26 +487,28 @@ static int
 os_map(void)
 {
 	struct cbdata data;
-	addrxlat_cb_t cb = {
-		.data = &data,
-		.get_page = get_page,
-		.read_caps = ADDRXLAT_CAPS(entry_as),
-		.sym = get_symdata
-	};
-	addrxlat_osdesc_t meth;
+	addrxlat_cb_t *cb;
+	addrxlat_opt_t opts[ADDRXLAT_OPT_NUM];
 	addrxlat_status status;
 
-	meth.type = ostype;
-	meth.ver = osver;
-	meth.arch = arch;
-	meth.opts = opts;
-
 	data.ctx = addrxlat_ctx_new();
 	if (!data.ctx) {
 		perror("Cannot allocate addrxlat");
 		return TEST_ERR;
 	}
-	addrxlat_ctx_set_cb(data.ctx, &cb);
+	cb = addrxlat_ctx_add_cb(data.ctx);
+	if (!cb) {
+		perror("Cannot allocate addrxlat callbacks");
+		return TEST_ERR;
+	}
+	cb->priv = &data;
+	cb->get_page = get_page;
+	cb->read_caps = read_caps;
+	cb->reg_value = get_symdata_reg;
+	cb->sym_value = get_symdata_value;
+	cb->sym_sizeof = get_symdata_sizeof;
+	cb->sym_offsetof = get_symdata_offsetof;
+	cb->num_value = get_symdata_number;
 
 	data.sys = addrxlat_sys_new();
 	if (!data.sys) {
@@ -422,7 +517,9 @@ os_map(void)
 		return TEST_ERR;
 	}
 
-	status = addrxlat_sys_os_init(data.sys, data.ctx, &meth);
+	status = addrxlat_sys_os_init(data.sys, data.ctx,
+				      make_opts(opts), opts);
+
 	if (status != ADDRXLAT_OK) {
 		fprintf(stderr, "OS map failed: %s\n",
 			addrxlat_ctx_get_err(data.ctx));
@@ -471,7 +568,7 @@ os_map(void)
 static int
 symheader(struct page_data *pg, char *p)
 {
-	addrxlat_sym_t *ss = pg->priv;
+	struct symstore *ss = pg->priv;
 	char *delim;
 	int argc;
 
@@ -482,19 +579,19 @@ symheader(struct page_data *pg, char *p)
 	}
 
 	if (!strncmp(p, "REG", 3)) {
-		ss->type = ADDRXLAT_SYM_REG;
+		ss->list = &symdata_reg;
 		p += 3;
 	} else if (!strncmp(p, "VALUE", 5)) {
-		ss->type = ADDRXLAT_SYM_VALUE;
+		ss->list = &symdata_value;
 		p += 5;
 	} else if (!strncmp(p, "SIZEOF", 6)) {
-		ss->type = ADDRXLAT_SYM_SIZEOF;
+		ss->list = &symdata_sizeof;
 		p += 6;
 	} else if (!strncmp(p, "OFFSETOF", 8)) {
-		ss->type = ADDRXLAT_SYM_OFFSETOF;
+		ss->list = &symdata_offsetof;
 		p += 8;
 	} else if (!strncmp(p, "NUMBER", 6)) {
-		ss->type = ADDRXLAT_SYM_NUMBER;
+		ss->list = &symdata_number;
 		p += 6;
 	} else {
 		fprintf(stderr, "Invalid symbolic header: %s\n", p);
@@ -509,7 +606,7 @@ symheader(struct page_data *pg, char *p)
 	}
 	++p;
 
-	for (argc = 0; argc < ADDRXLAT_SYM_ARGC_MAX; ++argc) {
+	for (argc = 0; argc < SYM_ARGC_MAX; ++argc) {
 		char *endp, *arg;
 
 		while (isspace(*p))
@@ -531,11 +628,11 @@ symheader(struct page_data *pg, char *p)
 		}
 		memcpy(arg, p, endp - p);
 		arg[endp - p] = '\0';
-		ss->args[argc] = arg;
+		ss->data.args[argc] = arg;
 
 		if (*delim == ')') {
-			while (++argc < ADDRXLAT_SYM_ARGC_MAX)
-				ss->args[argc] = NULL;
+			while (++argc < SYM_ARGC_MAX)
+				ss->data.args[argc] = NULL;
 			return TEST_OK;
 		}
 
@@ -549,7 +646,7 @@ symheader(struct page_data *pg, char *p)
 static int
 storesym(struct page_data *pg)
 {
-	addrxlat_sym_t *ss = pg->priv;
+	struct symstore *ss = pg->priv;
 	addrxlat_addr_t val;
 	size_t sz = pg->len;
 
@@ -557,19 +654,17 @@ storesym(struct page_data *pg)
 		sz = sizeof(val);
 	val = 0;
 #if __BYTE_ORDER == __LITTLE_ENDIAN
-	memcpy(&val, pg->buf, pg->len);
+	memcpy(&val, pg->buf, sz);
 #else
-	memcpy((char*)(&val + 1) - sz,
-	       pg->buf + sizeof(val) - pg->len,
-	       pg->len);
+	memcpy((char*)(&val + 1) - sz, pg->buf, sz);
 #endif
-	return add_symdata(ss, val);
+	return add_symdata(ss->list, ss->data.args, val);
 }
 
 static int
 read_sym(void)
 {
-	addrxlat_sym_t ss;
+	struct symstore ss;
 	struct page_data pg;
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
@@ -647,6 +742,7 @@ main(int argc, char **argv)
 	} else
 		param = stdin;
 
+	clear_params();
 	rc = parse_params_file(&params, param);
 	if (param != stdin)
 		fclose(param);
