@@ -28,12 +28,12 @@
 
 #define _GNU_SOURCE
 
+#include "testutil.h"
+
 #include <stdio.h>
 
 #include <libkdumpfile/addrxlat.h>
 
-#include "testutil.h"
-
 struct test {
 	addrxlat_fulladdr_t addr;
 	addrxlat_fulladdr_t expect;
@@ -198,35 +198,10 @@ static struct test test_nomach[] = {
 	},
 };
 
-static void
-print_addrspace(addrxlat_addrspace_t as)
-{
-	switch (as) {
-	case ADDRXLAT_KPHYSADDR:
-		fputs("KPHYSADDR", stdout);
-		break;
-
-	case ADDRXLAT_MACHPHYSADDR:
-		fputs("MACHPHYSADDR", stdout);
-		break;
-
-	case ADDRXLAT_KVADDR:
-		fputs("KVADDR", stdout);
-		break;
-
-	case ADDRXLAT_NOADDR:
-		fputs("NOADDR", stdout);
-		break;
-
-	default:
-		printf("<addrspace %ld>", (long) as);
-	}
-}
-
 static void
 print_fulladdr(const addrxlat_fulladdr_t *addr)
 {
-	print_addrspace(addr->as);
+	fputs(addrxlat_addrspace_name(addr->as), stdout);
 	if (addr->as != ADDRXLAT_NOADDR)
 		printf(":0x%"ADDRXLAT_PRIxADDR, addr->addr);
 }
